@@ -308,7 +308,6 @@ void cmdline(int argc, char *argv[])
 			case 'c': flags |=F_COPYONWRITE;
 			        break;
 			case 'l':
-				free(auth_file);
 				if (i+1<argc) {
 					auth_file=argv[++i];
 				} else {
@@ -344,6 +343,10 @@ void cmdline(int argc, char *argv[])
 	}
 
 	exportname = argv[2];
+	if(exportname[0] != '/') {
+		fprintf(stderr, "%s requires an absolute path name to the exported file\n", argv[0]);
+		exit(EXIT_FAILURE);
+	}
 }
 
 /**
@@ -671,7 +674,11 @@ int mainloop(int net)
 		request.from = ntohll(request.from);
 		request.type = ntohl(request.type);
 
-		if (request.type==NBD_CMD_DISC) { /* Disconnect request */
+		/* Disconnect request. 2.6 nbd.h defines NBD_CMD_DISC, which we
+		 * will use (again) at some point in the future, but many
+		 * people are still using 2.4-based systems, and they need to
+		 * be able to compile as well.*/
+		if (request.type==2) {
 		  if (difmap) free(difmap) ;
                   if (difffile>=0) { 
                      close(difffile) ; unlink(difffilename) ; }
